import "./NumberInput.css";
import { cssModule, forwardRef, isNil, mergeClasses, mergeProps, omitProps, useChainedEventCallback, useControllableState, useEventCallback } from "../../shared";
import { Box } from "../../box";
import { CaretIcon } from "../../icons";
import { useCallback } from "react";
import { useFieldInputProps } from "../../field";
import { useInput, useInputIcon, wrappedInputPropsAdapter } from "../../input";
import { useInputGroupProps } from "../../input-group";
import { useToolbarProps } from "../../toolbar"; // Used to generate BoxProps instead of any in the auto-generated documentation
// eslint-disable-next-line @typescript-eslint/no-empty-interface

import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
import { Fragment as _Fragment } from "react/jsx-runtime";

function Spinner({
  onIncrement,
  onDecrement,
  onFocus,
  disabled,
  ...rest
}) {
  const handleIncrement = useEventCallback(event => {
    onIncrement(event);
  });
  const handleDecrement = useEventCallback(event => {
    onDecrement(event);
  });
  return /*#__PURE__*/_jsxs("div", { ...mergeProps(rest, {
      className: "o-ui-number-input-spinner",
      "aria-hidden": true
    }),
    children: [/*#__PURE__*/_jsx("button", {
      onClick: handleIncrement,
      className: "o-ui-number-input-spinner-increment",
      type: "button",
      tabIndex: -1,
      disabled: disabled,
      onFocus: onFocus,
      "aria-label": "Increment value",
      children: /*#__PURE__*/_jsx(CaretIcon, {
        size: "xs"
      })
    }), /*#__PURE__*/_jsx("button", {
      onClick: handleDecrement,
      className: "o-ui-number-input-spinner-decrement",
      type: "button",
      tabIndex: -1,
      disabled: disabled,
      onFocus: onFocus,
      "aria-label": "Decrement value",
      children: /*#__PURE__*/_jsx(CaretIcon, {
        size: "xs",
        className: "o-ui-rotate-180"
      })
    })]
  });
}

function countDecimalPlaces(value) {
  return (value.toString().split(".")[1] || []).length;
}

function toNumber(value) {
  const result = parseFloat(value);

  if (isNaN(result)) {
    return null;
  }

  return result;
}

function toFixed(value, precision) {
  return parseFloat(value.toFixed(precision));
}

export function InnerNumberInput(props) {
  const [toolbarProps] = useToolbarProps();
  const [fieldProps] = useFieldInputProps();
  const [inputGroupProps, isInGroup] = useInputGroupProps();
  const contextualProps = mergeProps({}, omitProps(toolbarProps, ["orientation"]), fieldProps, inputGroupProps);
  const {
    id,
    value,
    defaultValue,
    placeholder,
    min,
    max,
    step = 1,
    required,
    validationState,
    onValueChange,
    onChange,
    autoFocus,
    icon,
    disabled,
    readOnly,
    fluid,
    loading,
    active,
    focus,
    hover,
    "aria-label": ariaLabel,
    "aria-labelledby": ariaLabelledBy,
    wrapperProps: wrapperPropsProp,
    as = "div",
    forwardedRef,
    ...rest
  } = mergeProps(props, wrappedInputPropsAdapter(contextualProps));

  if (isNil(ariaLabel) && isNil(ariaLabelledBy) && isNil(placeholder)) {
    console.error("An input component must either have an \"aria-label\" attribute, an \"aria-labelledby\" attribute or a \"placeholder\" attribute.");
  }

  const [inputValue, setValue] = useControllableState(value, defaultValue, null);

  const updateValue = (event, newValue) => {
    if (newValue !== inputValue) {
      if (!isNil(onValueChange)) {
        onValueChange(event, newValue);
      }

      setValue(newValue);
    }
  };

  const validateRange = useCallback(newValue => {
    let isAboveMax = false;
    let isBelowMin = false;

    if (!isNil(newValue)) {
      if (!isNil(max) && newValue > max) {
        isAboveMax = true;
      } else if (!isNil(min) && newValue < min) {
        isBelowMin = true;
      }
    }

    return {
      isInRange: !isAboveMax && !isBelowMin,
      isAboveMax,
      isBelowMin
    };
  }, [min, max]);

  const clampValue = (event, newValue) => {
    const {
      isInRange,
      isBelowMin,
      isAboveMax
    } = validateRange(newValue);

    if (isInRange) {
      updateValue(event, newValue);
    } else {
      if (isBelowMin) {
        updateValue(event, min);
      } else if (isAboveMax) {
        updateValue(event, max);
      }
    }
  };

  const applyStep = (event, factor) => {
    if (!isNil(inputValue)) {
      const precision = Math.max(countDecimalPlaces(inputValue), countDecimalPlaces(step));
      const newValue = toFixed(inputValue + factor * step, precision);
      clampValue(event, newValue);
    } else {
      updateValue(event, factor * step);
    }
  };

  const handleChange = useChainedEventCallback(onChange, (event, newValue) => {
    clampValue(event, toNumber(newValue));
  });
  const handleIncrement = useEventCallback(event => {
    applyStep(event, 1);
  });
  const handleDecrement = useEventCallback(event => {
    applyStep(event, -1);
  });
  const handleStepperFocus = useEventCallback(() => {
    inputRef.current.focus();
  });
  const {
    wrapperProps,
    inputProps,
    inputRef
  } = useInput({
    cssModule: "o-ui-number-input",
    id,
    value: !isNil(inputValue) ? inputValue : "",
    placeholder,
    required,
    validationState,
    onChange: handleChange,
    type: "number",
    autoFocus,
    disabled,
    readOnly,
    fluid,
    loading,
    active,
    focus,
    hover,
    forwardedRef
  });
  const iconMarkup = useInputIcon(icon, {
    disabled
  });

  const content = /*#__PURE__*/_jsxs(_Fragment, {
    children: [iconMarkup, /*#__PURE__*/_jsx("input", { ...mergeProps(rest, {
        "aria-label": ariaLabel,
        "aria-labelledby": ariaLabelledBy
      }, inputProps)
    }), /*#__PURE__*/_jsx(Spinner, {
      onIncrement: handleIncrement,
      onDecrement: handleDecrement,
      onFocus: handleStepperFocus,
      disabled: readOnly || disabled,
      "aria-hidden": loading
    })]
  });

  return /*#__PURE__*/_jsx(Box, { ...mergeProps(wrapperPropsProp, wrapperProps, {
      className: mergeClasses(cssModule("o-ui-input", iconMarkup && "has-icon", isInGroup && "in-group"), cssModule("o-ui-number-input", isInGroup && "in-group")),
      as
    }),
    children: content
  });
}
export const NumberInput = forwardRef((props, ref) => /*#__PURE__*/_jsx(InnerNumberInput, { ...props,
  forwardedRef: ref
}));
NumberInput.displayName = "NumberInput";